#include "global.h"
#include "keys.h"

typedef struct {
  uint8_t count:4;
  uint8_t status:4;
} KEY;

KEY keys[2]= {{ 0, KEY_RELEASED},
	      { 0, KEY_RELEASED}};

void
key_check() {
  uint8_t k;

  KEY1_PORT_OUT |= _BV(KEY1_PIN);
  KEY1_PORT_DDR &= ~_BV(KEY1_PIN);
  key_delay_2us();
  k=1;
  if (KEY1_PORT_IN & _BV(KEY1_PIN))
    k=0;
  key_status_ajust(KEY1, k);

  KEY2_PORT_OUT |= _BV(KEY2_PIN);
  KEY2_PORT_DDR &= ~_BV(KEY2_PIN);
  key_delay_2us();
  k=1;
  if (KEY2_PORT_IN & _BV(KEY2_PIN))
    k=0;
  key_status_ajust(KEY2, k);
}

void
key_status_ajust(uint8_t nr, uint8_t k)  {
  KEY key=keys[nr];  /* local value helps the compiler to make better code */

  if (k) {
    if (key.count < 15)
      key.count++;
  }
  else {
    if (key.count > 0)
      key.count--;
  }
  switch (key.status) {
  case KEY_RISE:
    key.status = KEY_PRESSED;
    break;
  case KEY_RELEASED:
    if (key.count == 15)
      key.status = KEY_RISE;
    break;
  case KEY_PRESSED:
    if (key.count == 0)
      key.status = KEY_FALL;
    break;
  case KEY_FALL:
  default:
    key.status = KEY_RELEASED;
  }
  keys[nr]=key;
}

uint8_t
key_rised(uint8_t key){
  return keys[key].status == KEY_RISE;
}

void
key_delay_2us() { /* needed because signals of the keys are too slow */
  uint8_t i;
  /* t = i * loop / UC_CLOCK 
     i = t / loop * UC_CLOCK
     the fastest way the compiler can do that loop takes 4 clocks:
     1 clocks for decrementing, 1 for nop, 2 clocks to branch.
     take care that i does not exceed max(uint8_t) */
  /* check the compiler, it may unroll that loop to nop; nop; nop; */
  for (i=UC_CLOCK*2e-6/4; i; i--)
    asm volatile("nop\n\t"::);   /* required to prevent compiler optimisation */
}
