/*****************************************************************************
project:  7segment clock
author:   Werner Hoch <werner.ho@gmx.de>
file:     main.c
*****************************************************************************/

#include <avr/signal.h>          /* interrupt vectors */
#include <avr/interrupt.h>       /* sei(), cli(),... */
#include <string.h>
#include "global.h"
#include "seg7.h"
#include "keys.h"

#define SUB_CLOCK   360L

uint8_t second=0, minute=58, hour=23;
uint16_t prescaler;

volatile uint8_t flag_sec=0, flag_halfsec=0;
volatile uint8_t flag_key1=0, flag_key2=0;

#define STATE_NORMAL 0
#define STATE_CHANGE_SEC 1
#define STATE_CHANGE_MIN 2
#define STATE_CHANGE_HOUR 3
uint8_t state=STATE_NORMAL;


SIGNAL 
(SIG_OUTPUT_COMPARE1A) {
#if UC_CLOCK % SUB_CLOCK
  OCR1A = UC_CLOCK / SUB_CLOCK - 1;          // compare SUB_CLOCK - 1 times
#endif
  if( --prescaler == 0 ){
    prescaler = SUB_CLOCK;
    flag_sec=1;
#if UC_CLOCK % SUB_CLOCK                     // handle remainder
    OCR1A = UC_CLOCK / SUB_CLOCK + UC_CLOCK % SUB_CLOCK - 1; // compare once per second
#endif
  }
  if (prescaler == SUB_CLOCK/2)
    flag_halfsec=1;

  seg7_off();
  key_check();
  flag_key1=key_rised(KEY1);
  flag_key2=key_rised(KEY2);
  seg7_multiplex();
}

int
main() {
  seg7_init();
  /* seg7_test(); */

  TCCR1B = _BV(CTC1) | _BV(CS10); /* clear on compare match, prescaler 1 */

  OCR1A = UC_CLOCK / SUB_CLOCK - 1;
  TCNT1 = 0;
  prescaler = SUB_CLOCK;
  
  TIMSK = _BV(OCIE1A);
  sei();

  while (1) {  /* infinite loop */
    if (flag_halfsec){
      flag_halfsec=0;
      switch (state){
      case STATE_NORMAL: 
	break;
      case STATE_CHANGE_HOUR: 
	seg7_data_set(0,0);
	seg7_data_set(1,0);
	break;
      case STATE_CHANGE_MIN:
	seg7_data_set(2,0);
	seg7_data_set(3,0);
	break;
      case STATE_CHANGE_SEC:
	seg7_data_set(4,0);
	seg7_data_set(5,0);
	break;
      }
      seg7_set_dots();
    }
    if (flag_sec) {
      flag_sec=0;
      if (++second == 60){
	second=0;
	if (++minute == 60) {
	  minute=0;
	  if (++hour == 24) {
	    hour=0;
	  }
	}
      }
      seg7_dec_set(0,hour);
      seg7_dec_set(2,minute);
      seg7_dec_set(4,second);
      seg7_clear_dots();
    }
    if (flag_key1) {
      flag_key1=0;
      switch (state){
      case STATE_NORMAL: state=STATE_CHANGE_HOUR; break;
      case STATE_CHANGE_HOUR: state=STATE_CHANGE_MIN; break;
      case STATE_CHANGE_MIN: state=STATE_CHANGE_SEC; break;
      case STATE_CHANGE_SEC: state=STATE_NORMAL; break;
      default: state=STATE_NORMAL;
      }
    }
    if (flag_key2) {
      flag_key2=0;
      switch (state){
      case STATE_CHANGE_HOUR: 
	if (++hour==24)
	  hour=0;
	seg7_dec_set(0,hour);
	break;
      case STATE_CHANGE_MIN:
	if (++minute == 60)
	  minute=0;
	seg7_dec_set(2,minute);
	break;
      case STATE_CHANGE_SEC:
	TCNT1 = 0;
	OCR1A = UC_CLOCK / SUB_CLOCK - 1;
	prescaler = SUB_CLOCK;
	second=0;
	seg7_dec_set(4,second);
	break;
      default: 
	state=STATE_NORMAL;
      }
    }
  }
}
