#include "global.h"
#include "seg7.h"
#include <avr/delay.h>

const uint8_t PROGMEM DATA_TABLE[] = {SEG_CHAR_0, SEG_CHAR_1, SEG_CHAR_2,
				       SEG_CHAR_3, SEG_CHAR_4, SEG_CHAR_5,
				       SEG_CHAR_6, SEG_CHAR_7, SEG_CHAR_8,
				       SEG_CHAR_9};
const uint8_t PROGMEM DIGIT_TABLE[] = {DIGIT_0, DIGIT_1, DIGIT_2,
				       DIGIT_3, DIGIT_4, DIGIT_5};

uint8_t seg7_data[SEG7_DIGITS];


void
delay_ms(uint16_t i){
  for (;i;i--)
    _delay_loop_2(UC_CLOCK/4*0.001);
}


/* clear the display */
void 
seg7_init() {
  uint8_t i;
  DIGIT_DDR |= DIGIT_MASK;  /* define as output */
  DIGIT_PORT |= DIGIT_MASK; /* clear relevant pins */
  DATA_DDR |= 0xFF;         /* define as output */
  DATA_PORT |= 0xFF;        /* clear relevant pins */
  for (i=0;i<SEG7_DIGITS;i++)
    seg7_data[i]=0;
}

/* hardware tests */
void
seg7_test() {
  uint8_t i,j;
  uint16_t k;
  /* activate any segment separate */
  for (i=0x01;i;i=i<<1){
    for (j=0;j<6;j++) {
      seg7_data_set(j,i);
      seg7_multiplex();
      delay_ms(500);
    }
  }
  /* show "012345" sequently on the display */
  for (i=0; i<6;i++)
    seg7_digit_set(i,i);
  for (i=0; i<6;i++) {
    seg7_multiplex();
    delay_ms(1000);
  }
  /* 5 seconds of multiplex test */
  for (k=2500;k;k--) {
    delay_ms(2);
    seg7_multiplex();
  }
  seg7_dec_set(0,98);
  seg7_dec_set(2,76);
  seg7_dec_set(4,54);
  /* 5 seconds of multiplex test */
  for (k=2500;k;k--) {
    delay_ms(2);
    seg7_multiplex();
  }
}

/* multiplex to the next digit */
void 
seg7_multiplex() {
  static uint8_t digit=0;

  if (digit == SEG7_DIGITS)
    digit=0;

  DIGIT_DDR |= DIGIT_MASK;  /* define as output */
  DIGIT_PORT |= DIGIT_MASK; /* clear relevant pins */
  DATA_DDR |= 0xFF;         /* define as output */

  DIGIT_PORT &=~PRG_RDB(&DIGIT_TABLE[digit]);
  DATA_PORT =~seg7_data[digit];
  digit++;
}

/* display data on the display on place digitnr */
void 
seg7_data_set(uint8_t digitnr, uint8_t data) {
  seg7_data[digitnr]= data;
}

/* display a digit (0 to F) on the display on place digitnr */
void 
seg7_digit_set(uint8_t digitnr,uint8_t digit) {
  seg7_data[digitnr]= PRG_RDB(&DATA_TABLE[digit]);
}

void
seg7_dec_set(uint8_t digitnr, uint8_t data) {
  seg7_data[digitnr]=PRG_RDB(&DATA_TABLE[data/10]);
  seg7_data[digitnr+1]=PRG_RDB(&DATA_TABLE[data%10]);
}

void
seg7_set_dots(){
  seg7_data[0] |= SEG_DOT;
  seg7_data[1] |= SEG_DOT;
  seg7_data[2] |= SEG_DOT;
  seg7_data[3] |= SEG_DOT;
}

void
seg7_clear_dots(){
  seg7_data[0] &=~ SEG_DOT;
  seg7_data[1] &=~ SEG_DOT;
  seg7_data[2] &=~ SEG_DOT;
  seg7_data[3] &=~ SEG_DOT;
}

void 
seg7_off() {
  DIGIT_DDR &=~ DIGIT_MASK;  /* define as inputs */
  DIGIT_PORT &=~ DIGIT_MASK; /* turn pullups off */
  DATA_DDR = 0x00;         /* define as input */
  DATA_PORT = 0x00;        /* turn pullups off */
}
